###################### File medea.R ################################
# medea stands for MEasures Ascent/Descent
# and contains a code in R realising the steepest descent with
# variable step size for optimization problems in the space of
# positive measures with constraints.
# Theoretical background can be found in I. Molchanov, S. Zuyev
# "Steepest Descent Algorithms in Space of Measures" Preprint, 2000
######################################################################
# Version: 1.4
# Date:    24 October 2012
# Authors: 
#   Ilya Molchanov (ilya@stat.gla.ac.uk)
#   Sergei Zuyev   (sergei@stams.strath.ac.uk)
# Licence: GPL-2
######################################################################
####################### The code begins here #########################


#### SOME TYPICAL CONSTRAINTS ####################
const.default <- function(x,...){
# The only constraint on the total mass
	return(c(1))
	}
bary <- function(x,...){
# Constraints on the total mass and the barycentre
	return(c(1,x))
	}
bary.only <- function(x,adj=0,...){
# Barycentre only
# adj determines adjustment sometimes needed to avoid overflow 
	return(c(x))
	}
bary.sq <- function(x,...){
# The total mass, barycentre and the second moment
	return(c(1,x,x*x))
	}
bary.higher <- function(x,...){
# Barycentre and the second moment
	return(c(x,x*x))
	}
##################################################

############ AUXILIARY FUNCTIONS ##################
kill.neg <- function(x){
# Auxiliary function used to kill "bad" lines in a matrix
	if(min(x,na.rm=TRUE)<0 | any(is.na(x)) | max(abs(x))==Inf )
		{x <- rep(NA,length(x))}
	return(x)
}

space.to.grid <- function(space,nv)
{ # Auxiliary function used to precalculate constraints
	len <- length(space[,1])
	c.l <- list(1:len)
	if(nv > 1){
		for(i in 2:nv){
			c.l <- c(c.l,list(1:len))
		}}
	res <- data.matrix(expand.grid(c.l))
	result <- t(apply(res,1,get.space,space=space))
	return(result)
}

get.space <- function(x,space)
{ # Auxiliary function used to prepare the space X^k 
	nd <- length(x)
	c.l <- as.vector(space[x[1],])
	if(nd >1){
		for (i in 2:nd){
			c.l <- c(c.l,space[x[i],])
		}}
	return(c.l)
}

inv.gen <- function(d.mat,v,...)
{ # Inversion of a general matrix of constraints
	nv <- length(v)
	big.des <- space.to.grid(d.mat,nv)
	if(nv==1){
		tr <- matrix(1/as.vector(d.mat))
		}
	if(nv==2){
		a1 <- as.vector(big.des[,1])
		b1 <- as.vector(big.des[,2])
		a2 <- as.vector(big.des[,3])
		b2 <- as.vector(big.des[,4])
		mm <- a1*b2-a2*b1
		tr <- cbind((v[1]*b2-v[2]*a2)/mm,
			(v[2]*a1-v[1]*b1)/mm,1:length(mm))
		}
	if(nv==3){
		a1 <- as.vector(big.des[,1])
		b1 <- as.vector(big.des[,2])
		c1 <- as.vector(big.des[,3])
		a2 <- as.vector(big.des[,4])
		b2 <- as.vector(big.des[,5])
		c2 <- as.vector(big.des[,6])
		a3 <- as.vector(big.des[,7])
		b3 <- as.vector(big.des[,8])
		c3 <- as.vector(big.des[,9])
		mm <- a1*b2*c3+a2*b3*c1+b1*c2*a3-a3*b2*c1-a1*c2*b3-a2*b1*c3
		mm[abs(mm)<1e-10]<-0
		r1 <- v[1]*(b2*c3-b3*c2)+v[2]*(c2*a3-a2*c3)+v[3]*(a2*b3-a3*b2)
		r2 <- v[1]*(b3*c1-b1*c3)+v[2]*(a1*c3-c1*a3)+v[3]*(b1*a3-a1*b3)
		r3 <- v[1]*(b1*c2-b2*c1)+v[2]*(c1*a2-a1*c2)+v[3]*(b2*a1-b1*a2)
		tr <-  cbind(r1/mm,r2/mm,r3/mm,1:length(mm))
		}
	if(nv>=4){
		stop("Inversion for 4-dimensional constraints is not yet implemented")
		}
	tr1 <- apply(tr,1,kill.neg)
	tr <- as.matrix(na.omit(as.data.frame(t(tr1))))
	return(tr)
	}

inv.constr <- function(x,values,constr,...)
{ # Inversion of constraints row by row (non-symbolic)
#    hco <- matrix(constr(space[ik[1],],...),ncol=1)
#    if(length(ik)>1)
#      {
#        for (i in 2:length(ik))
#          {
#            hco <- cbind(hco,constr(space[ik[i],],...))
#          }
#      }
    nv <- length(values)
    hco <- apply(matrix(x,nrow=nv),1,constr,...)
    if(abs(det(hco))>1e-10)
      {
        tr <- solve(hco,values)
        if(min(tr,rm.na=TRUE)<0){tr <- rep(NA,length(values))}
      }
    else{tr <- rep(NA,length(values))}
    return(tr)
  }

prepare.design <- function(space,constr,...)
{ # Calculation of the design matrix
  # The design matrix is calculated automatically in go.steep
  # or may be passed as d.mat argument
	 nv <- length(constr(space[1,]))
	 hs <- apply(space,1,constr,...)
	 if(is.matrix(hs)){hs <- t(hs)}
	 else {hs <- matrix(hs)}
	 return(hs)
}

prepare.constr <- function(space,constr,values,d.mat,...)
{ # This inverts matrix of constraints with rows that provide 
  # only non-negative masses
  # This matrix is calculated automatically in go.renorm
  # or may be passed as c.con argument	
  # If number of constrints <=3, inv.gen is applied, otherwise inv.constr
	nv <- length(values)
	all.grid <- space.to.grid(space,nv)
	if(nv<4){
		th <- inv.gen(d.mat,values,...)}
	else{
		th <- apply(all.grid,1,inv.constr,
			constr=constr,values=values,...)
		th1 <- rbind(th,1:length(th[1,]))
		th <- as.matrix(na.omit(as.data.frame(t(th1))))
	 	}
	return(t(th))
}

by.div <- function(x,n,nv)
{ # Auxiliary function used to determine the design point that
  # corresponds to a particular row in X^k
	g <- rep(0,nv)
	i <- 0
	while(x>0){
		i <- i+1
		g[i] <- x%%n
		x <- x%/% n 
		}
	return(g+1)
}
##################################################

############ FUNCTIONS USED TO PERFORM ###########
################ A TYPICAL STEP###################

is.optim.renorm<-function(tol,mu,g,space,des.mat) 
 {# Function checking necessary condition for a minimum
   tol.supp<-tol*mean(mu)
   reg <- glm(g~des.mat-1, weights=mu)
   tfit <- as.vector(fitted(reg))
   range.fit <- tol*abs(max(tfit)-min(tfit))
   mtd<-min(g-tfit) # now tfit+mtd is a lower bound for g
   var.gr<-max((g-tfit-mtd)[mu>tol.supp]) # var of grad on supp(mu)
   tfit1 <- tfit+mtd#+range.fit
   tt<-(var.gr<range.fit) & (min(g-tfit1) >= -range.fit)
   return(list(opt=tt,var=var.gr,fit=tfit1))
 }

locations <- function(g,cspace,...)
{ # Determines locations of points and their masses to be added on every 
  # step to preserve the constraints on the measure
  nv <- length(cspace[,1])-1
  n.no <- as.vector(cspace[nv+1,])
  gl <- list(g)
  if(nv>1){for (i in 2:nv){gl <- c(gl,list(g))}}
  gex <- data.matrix(expand.grid(gl))
  geh <- apply(t(gex[n.no,])*cspace[1:nv,],2,sum)
  loc <- matrix(geh==min(geh,na.rm=TRUE))
  if(all(is.na(loc))){stop("Problems finding locations")}
  loc[is.na(loc)]<-FALSE
  loc0 <- row(loc)[loc][1]
  loc1 <- n.no[loc0]
  p <- as.vector(cspace[1:nv,loc0])
  loc.found <- as.vector(by.div(loc1-1,length(g),nv))
  return(list(loc=loc.found,mass=p,ass=loc0,bas=loc1))
}

take.step.renorm <- function(m,mu,g,cspace,...)
{# Function putting  mass m to where g is minimal
   m <- min(c(m,1))
   locat <- locations(g,cspace,...)
   loc <- locat$loc
   p <- locat$mass
   mu1 <- mu
   mu1[loc]<-mu1[loc]+p*m
 # Now take mass a mass proportional to mu itself 
 # (equivalent to renormalising)
   mu2<-mu1-m*mu
   return(mu2)
 }
##################################################

######### GO RENORM PART ##########################

go.renorm<-function(object, ...)
  {
    if(is.null(class(object))) class(object)<-data.class(object)
    UseMethod("go.renorm")
  }

go.renorm.medea<-function(med,
                 obj.func.name=med$obj.func$name,
                 grad.func.name=med$grad.func$name,
                 minim=med$minim,fval=med$fval,
                 grad=med$grad,tol=med$param$tol,st.size=med$st.size,
                 No.steps=100,space=med$param$space,
                 details=FALSE,visualize=FALSE,smallest.st.size=0.000001,
                 draw.style=4,xgrid=med$param$xgrid,ygrid=med$param$ygrid,
                 tmp.file=med$param$file,
                 base=med$param$base,uni=med$param$uni,
		 c.con=med$c.con,d.mat=med$d.mat,
		 constr=med$param$constr,cval=med$param$cval,
                 true.m=med$true.m,...)
  {
    ar<-c(list(mu=med$mes,obj.func=med$obj.func$code,
               grad.func=med$grad.func$code,
               obj.func.name=obj.func.name,grad.func.name=grad.func.name,
               minim=minim,fval=fval,grad=grad,tol=tol,
               st.size=st.size,No.steps=No.steps,space=space,
               details=details,visualize=visualize,
               smallest.st.size=smallest.st.size,
                     draw.style=draw.style,xgrid=xgrid,ygrid=ygrid,
                     tmp.file=tmp.file,base=base,uni=uni,
	       constr=med$param$constr,cval=med$param$cval,
	       c.con=med$c.con,d.mat=med$d.mat,
               dim=med$param$dim,
                     true.m=true.m),med$other,...=...)
    do.call("go.renorm.default",ar)
  }

go.renorm.default<-function(mu,obj.func,grad.func,
                           obj.func.name=NA,grad.func.name=NA,
                           minim=TRUE,fval=NA,grad=NA,tol=0.01,
                           st.size=NA,No.steps=100,space=NA,
                           details=FALSE,visualize=FALSE,
                           smallest.st.size=0.000001,
                           draw.style=4,xgrid=NA,ygrid=NA,
                           tmp.file=NA,base=0.5,
			   c.con=NA,d.mat=NA,
			   constr=NA,cval=NA,
                           uni=FALSE,dim=NA,true.m=NA,nlevels=5,ncolors=0,...)
{ 
# The main function: minimizes (if minim=TRUE) or maximizes objective
# function with the name obj.func. The gradient function is grad.func,
# fval is the initial value of obj.func if known, true.m is the true
# minimal (maximal) value of obj.func, if known.
# These functions are functions on measures on the state space space,
# space itself
# and possibly some other parameters that should be listed in ... part.
# If space is not supplied then it is taken to be 1:length(mu).
# Algorithm goes at most No.steps steps until the measure mu satisfies
# the necessary optimality condition or reaches tru.m up to error tol.
# Returns object of class medea.
# A step i consists in changing the current measure mu.cur by a measure eta
# along the gradient grad with the total mass 0 and the total
# variation st.size.cur.
# If the value of objective function in the changed measure do not decrease
# and then st.size.cur is changed to base*st.size.cur.
# Other parameters:
# tol - the tolerance when deciding if the gradient function is a constant
# on the support of the measure mu. Used in is.optim.renorm() function.
# If details=TRUE, the values of st.size and obj.func(mu) at each step
# are printed.
# If visualize=TRUE the graphs of the measure and the gradient are drawn
# after each step of the algorithm.
# For 2D state space draw.style is 0 for image(), 1 for persp(). If 2, the
# state space will be shown as a vector - useful to see how close to the
# minimum you are.
# In computationally extensive cases it is advisable to set tmp.file to
# a string (name of the file) to store the currenly obtained mu. You then
# will be able to retrieve it with mu<-dget("<name.of.tmp.file>") and
# reuse it if the program interrupts.
# The algorithm adds a positive mass where the gradient function is minimal.
# If there are a few global minima and uni=TRUE then the mass is uniformly
# distributed among them. Otherwize the mass is added in just one point.
#
# First of all, check consistency of the parameters
if(!is.vector(mu) & !all(is.na(mu)))
  stop("the first parameter should be a vector")
if(!is.function(obj.func))
  stop("'obj.func' should be a function")
if(!is.function(grad.func))
  stop("'grad.func' should be a function")
if(!is.logical(minim))
  stop("'minim' should be logical")
if(!tol>0)
  stop("'tol' should be positive")
if((!st.size>0) & (!is.na(st.size)))
  stop("'st.size' should be positive")
#if(st.size>=sum(mu) & !all(is.na(mu)))
#  stop("'st.size' should be less than the total mass of the measure")
if(!No.steps>0)
  stop("'No.steps' should be positive")
if(!is.logical(details))
  stop("'details' should be logical")
if(!is.logical(visualize))
  stop("'visualize' should be logical")
if(!smallest.st.size>0)
  stop("'smallest.st.size' should be positive")
if(!is.character(tmp.file) & !is.na(tmp.file))
  stop("'tmp.file' should be a character string")
if(!(base>0 & base<1))
  stop("'base' should be in (0,1)")

if(is.na(obj.func.name) || !is.character(obj.func.name))
    obj.func.name<-paste("'",deparse(substitute(obj.func)),"'",sep="")
    
if(is.na(grad.func.name) || !is.character(grad.func.name))
  grad.func.name<-paste("'",deparse(substitute(grad.func)),"'",sep="")
if(all(is.na(space)))
  {
    if(all(is.na(xgrid))||all(is.na(ygrid)))
      {
	if(any(is.na(mu))){stop("neither space nor mu is supplied")}
	else{
        	space<-as.matrix(1:length(mu))
        	warning(paste("State space is not supplied - considered to be 1:length(",deparse(substitute(mu)),").",sep=""))
      	}}
    else
      space<-as.matrix(expand.grid(xgrid,ygrid))
  }

if(!is.matrix(space)) space<-as.matrix(space)
if(is.na(dim))  dim<-length(space[1,])

if(all(is.na(d.mat))) d.mat <- prepare.design(space,constr,...)
if(!any(is.na(cval)) & all(is.na(c.con)))
	{c.con <- prepare.constr(space,constr,cval,d.mat,...)}

if(any(is.na(cval)))
  {if(all(is.na(mu)))
	   {stop("either measure or values must be specified")}
   else{
	cval <- as.vector(matrix(mu,nrow=1)%*%d.mat)
	if(all(is.na(c.con))) 
		{c.con <- prepare.constr(space,constr,cval,d.mat,...)}
	}
  }
else{
  if(all(is.na(mu))){
	fake.grad <- runif(length(space[,1]))
	tmpu <- locations(fake.grad,c.con)
#	tmpu <- locations(rep(1,length(space[,1])),c.con)
	mu <- rep(0,length(space[,1]))
	mu[tmpu$loc] <- tmpu$mass
#	cat("Initial messure \n",mu,"\n")
	}
  else{
	values.check <- as.vector(matrix(mu,nrow=1)%*%d.mat)
  	if(max(abs(values.check-cval))>1e-10)
  	{stop("the initial measure does not satisfy the constraints")}
	}
  }

if(is.na(st.size)){st.size <- 0.1*sum(mu)}

if(length(mu)!=length(space[,1]))
   stop("Lengths of the measure and the state space do not match")

min.ind<-1
if (!minim)
  min.ind<- -1 # indicator of minimization problem

# Getting the value of the goal function
#	
if(is.na(fval)){F0<-min.ind*obj.func(mu,space,...)}
else{F0<-min.ind*fval}
F0.init<-min.ind*F0
###Fval<-c(F0) ### values of obj.func
###Steps<-c(st.size) ### step sizes
# Tolerance of the support of measure
tol.supp<-tol*mean(mu)


# getting the gradient grad
if(all(is.na(grad)))
  grad<-grad.func(mu,space,...)

# The following part aims to deal with the case when the offered initial
# measure causes gradient to blow
g.check <- (max(abs(grad))==Inf)
while(g.check){
  #cat(mu,"\n")
  tmpg <- runif(length(space[,1]))
  mu <- take.step.renorm(st.size,mu,tmpg,c.con,...)
  grad <- grad.func(mu,space,...)
  g.check <- (max(abs(grad))==Inf)
}

grad<-min.ind*grad

######################################################################
# 
true.reach <- FALSE
if(!is.na(true.m)){true.reach <- (abs(F0-true.m)<tol)}
opt.chk<-is.optim.renorm(tol,mu,grad,space,d.mat)
is.optim<- opt.chk$opt | true.reach

if (is.optim)
  { # Collect some info at least
    supp<-as.matrix(space[mu>tol.supp,])
    mu.supp<-mu[mu>tol.supp]
    No.atoms<-sum(rep(1,length(mu))[mu>tol.supp])
    supp.str<-paste("\nThe current measure contains ",
                    No.atoms," atoms:","\n\n",sep="")
    for (j in 1:length(supp[,1]))
      {
        coor<-"("
        for (k in 1:dim)
          coor<-paste(coor,as.character(supp[j,][[k]]),",",sep="")
        coor<-paste(coor,")",sep="")
        supp.str<-paste(supp.str,"mu",coor,"=",format(mu.supp[[j]],digits=6),
                        "\n",sep="")
      }
    if(true.reach)
      cat("The true extremal value",true.m,"is attained within error tol=",tol)
    else
      cat("The measure already satisfies the optimality condition\n")
    invisible(return(structure(list(mes=mu,
                                obj.func
                                    =list(name=obj.func.name,code=obj.func),
                                grad.func
                                    =list(name=grad.func.name,code=grad.func),
                                    minim=minim,fval=min.ind*F0,
                    grad=min.ind*grad,var.grad=opt.chk$var,optim=TRUE,
		    st.size=st.size,
                    atoms=supp.str,No.atoms=No.atoms,tot.steps=0,
                    true.m=true.m,### Fval=Fval, Steps=Steps,###
		    c.con=c.con,d.mat=d.mat,
		    fits=opt.chk$fit,
                    param=list(tol=tol,init.st.size=st.size,
                    smallest.st.size=smallest.st.size,uni=uni,space=space,
                    xgrid=xgrid,ygrid=ygrid,dim=dim,
		    constr=constr,cval=cval,
                    base=base,file=tmp.file),other=list(...)),
                                      class="medea")))
                                     
  }
i<-1  # number of steps along the gradient
st.size.cur<-st.size # the current step size
too.small.step.size<-FALSE

if (!details) cat("\nSteps done:   ")
  i0<-0 # the total number of incremental steps
  while (i<=No.steps & !is.optim & !too.small.step.size)
    { # while i<=No.steps loop
      i0.change<-i0   #remember the step at which the direction changes
      is.smaller<-TRUE   #initialisation of the while-loop below:
      while (is.smaller)
        { # go in the direction grad while obj.func(mu) diminishes
                                      # Now try an incremental step:
          mu.new<-take.step.renorm(st.size.cur,mu,grad,c.con,...)
          F1<-min.ind*obj.func(mu.new,space,...)
          if (F1<F0)   # obj.func has decreased, update mu
            { 
              mu<-mu.new
              i0<-i0+1 # incremental step was done
              F0<-F1
            }
          else  # obj.func has not decreased - end the while loop
            {
              is.smaller<-FALSE                
              if (i0.change==i0) # not only we can not make a step
                {               # but we have not done any step at all: 
                  st.size.cur<-st.size.cur*base # decrease the st.size 
                  i<-(i-1)		    # do not count this step
                  if(!details)
                    cat("-")
                  if (st.size.cur<smallest.st.size)
                    too.small.step.size<-TRUE
                }
              else # increase st.size if too many incremental steps
                # was made with the current st.size
                {
                  st.size.cur<-st.size.cur*(i0-i0.change)
                  if(!details)
                    cat(paste(rep("+",i0-i0.change-1),sep="",collapse=""))
                }
            }
        } # end of while(is.smaller) cycle
      ###Fval<-c(Fval,F0) ### add new fval
      ###Steps<-c(Steps,st.size.cur) ###
      if(!is.na(tmp.file))  # tmp.file is set - write the current measure
        dput(mu,tmp.file)
      grad<-min.ind*grad.func(mu,space,...) # Take a new gradient
      if(!is.na(true.m))
        true.reach <- (abs(F0-true.m)<tol)
      opt.chk<-is.optim.renorm(tol,mu,grad,space,d.mat)
      is.optim<-opt.chk$opt | true.reach
                                      #if TRUE the loop finishes
      if (details)
                {
                  cat("Step:",i,"  ","St.size:",st.size.cur,"\t")
                  cat("obj.func=",min.ind*F0,"  ",
                      "var.grad=",opt.chk$var,"\n") 
                }
              else
                {
                  cat(".")
                }
      if(visualize) 
        {
          draw.medea(mu,min.ind*grad,opt.chk$fit,minim,
      	draw.style=draw.style,space,xgrid,ygrid,nlevels,ncolors)
        }
      i<-i+1
    } # end of while(i<=No.steps...) loop

# The descent is over now. Print the current results
if (!details)
  cat("\n")
if (is.optim) 
  { 
    if(true.reach)
      cat("THE TRUE EXTREMAL VALUE IS REACHED.")
    else
      cat("\n","OPTIMUM IS ATTAINED.","\n")
  }
else
  { 
    cat("\n","OPTIMUM IS NOT YET ATTAINED.","\n\n")
    cat("Consider rerunning go.renorm() and possibly change the parameters.\n")
    if (too.small.step.size)
      {cat("COMPUTATION STOPPED - TOO SMALL STEP SIZE:",st.size.cur,"\n")}
  }
# Now generate string to print mu in its support points only
supp<-as.matrix(space[mu>tol.supp,])
mu.supp<-mu[mu>tol.supp]
No.atoms<-sum(rep(1,length(mu))[mu>tol.supp])
supp.str<-paste("\nThe obtained measure contains ",
                No.atoms," atoms",
                "\n(you might consider collapsing close ones):",
                "\n\n",sep="")
for (j in 1:length(supp[,1]))
  {
    if(dim==1)
      coor<-paste("(",as.character(supp[j,][[1]]),")",sep="")
    else
      {
        coor<-"("
        for (k in 1:dim)
          coor<-paste(coor,as.character(supp[j,][[k]]),",",sep="")
        coor<-paste(coor,")",sep="")
      }
    supp.str<-paste(supp.str,"mu",coor,"=",format(mu.supp[[j]],digits=6),
                    "\n",sep="")
  }
cat("\nValue of",obj.func.name,"was:",F0.init,
    "  now:",min.ind*F0,"\n")
if (details)
  {
    if (No.atoms<31 || is.optim)
      {
        cat("\n","Current measure:","\n")
        cat(supp.str)
      }
    else
      {
        cat("\nCurrent measure contains",No.atoms,"atoms: ")
        cat("the output is suppressed." )
        cat("\nTo see the support points use \n\tcat(<medea_obj>$atoms)\n")
      }
    cat("\nVariation of the gradient on the measure's support =",opt.chk$var,"\n")

  # This may serve as an indication of how close we are to the optimum
#    cat("Total No. of incremental steps made:",i0,"\n")
  }
invisible(return(structure(list(mes=mu,
                                obj.func
                                =list(name=obj.func.name,code=obj.func),
                                grad.func
                                =list(name=grad.func.name,code=grad.func),
                                minim=minim,fval=min.ind*F0,
                                grad=min.ind*grad,var.grad=opt.chk$var,
                                optim=is.optim,st.size=st.size.cur,
                                atoms=supp.str,No.atoms=No.atoms,
                                tot.steps=i-1,true.m=true.m,
                                ###Fval=Fval,Steps=Steps,
				c.con=c.con,d.mat=d.mat,
				fits=opt.chk$fit,
                                param=list(tol=tol,
                                init.st.size=st.size,
                                smallest.st.size=smallest.st.size,
                                uni=uni,space=space,
                                xgrid=xgrid,ygrid=ygrid,dim=dim,
				constr=constr,cval=cval,
                                base=base,file=tmp.file),
                                other=list(...)),
                           class="medea")))
}
##################################################

############# SUMMARIES AND PLOTS ################

print.medea<-function(med,...)
  {
    mf<-med
    if (mf$minim)
      cat("\n   Minimization of function",mf$obj.func$name,"\n")
    else
      cat("\n   Maximization of function",mf$obj.func$name,"\n")
    med<-unlist(c(format(med[5],digits=8),med[8],med[[18]][1],
                  format(med[7],digits=5)))
    NextMethod("print")
    invisible(mf)
  }

summary.medea<-function(med)
  structure(med, class=c("sum.medea",class(med)))

print.sum.medea<-function(med,...)
  {
    if (med$minim)
      cat("\n    Minimization of function",med$obj.func$name,
          "in dimension",med$param$dim,"\n")
    else
      cat("\n    Maximization of function",med$obj.func$name,
          "in dimension",med$param$dim,"\n")
    cat("with gradient function",med$grad.func$name,".\n")
#    cat("\nFunction called:\n")
#    print(med$call)
    cat("\nProgram was called with tolerance",med$param$tol)
    cat("\n\tand initial step size",med$param$init.st.size)
    cat("\nFinal step size:",med$st.size,"\n")
    if (med$optim) 
      {
        if(!is.na(med$true.m) && abs(med$fval-med$true.m)<med$param$tol)
          cat("TRUE EXTREMUM",med$true.m,"IS ATTAINED WITHIN ERROR",
              med$param$tol)
        else
          cat("\n","OPTIMUM IS ATTAINED.","\n")
         if (med$No.atoms<31)
          {
            cat("\n","OPTIMAL MEASURE:","\n")
            cat(med$atoms)
          }
        else
          {
            cat("\nOptimal measure contains",med$No.atoms,"atoms: ")
            cat("the output is suppressed." )
            cat("\nTo see the support points use")
            cat("\n\tcat(<medea_obj>$atoms)\n")
          }
      }
    else
      { 
        cat("\n","OPTIMUM IS NOT YET ATTAINED.","\n\n")
        cat("Consider rerunning go.renorm()")
        cat("\nand possibly change the parameters.","\n")
        if (med$No.atoms<31)
          {
            cat("\n","Current measure:","\n")
            cat(med$atoms)
          }
        else
          {
            cat("\nCurrent measure contains",med$No.atoms,"atoms:")
            cat("\nOutput is suppressed." )
            cat("\nTo see the support points use cat(<medea_obj>$atoms)")
            #cat("\nTo see the support points use cat(",list(med),"$atoms)") 
          }
      }
    # Now print some statistics
    cat("\n\nValue of the objective function",med$obj.func$name,
        ":",med$fval,"\n")
    cat("\nVariation of the gradient on the measure's support:",med$var.grad)
    cat("\nTotal No. of steps made during the last call:",med$tot.steps,"\n")
    invisible(med)
  }    

plot.medea<-function(med,draw.style=NA,space=med$param$space,xgrid=med$param$
xgrid,ygrid=med$param$ygrid,nlevels=15,...)
  {
    if(med$param$dim==1 && all(is.na(space)))
      warning("'space' is not supplied - considered to be 1:length(measure)")
    if(med$param$dim>2)
      warning("space considered to be 1:length(measure)")
    if(med$param$dim==2 && (all(is.na(xgrid)) || all(is.na(ygrid))))
      {
	if(!all(is.na(space)))
          {
            xgrid <- as.numeric(names(table(space[,1])))
            ygrid <- as.numeric(names(table(space[,2])))
          }
      }
    if(is.na(draw.style))
      {
        draw.style<-0
        if(med$param$dim==2) draw.style<-4
      }
    draw.medea(med$mes,med$grad,med$fits,med$minim,draw.style,space,
		xgrid,ygrid,nlevels,...)
    invisible(med)
  }

draw.medea<-function(mu,g,fits,minim=TRUE,draw.style=0,space=NA,xgrid=NA,ygrid=NA,
                   nlevels=15,ncolors=0,...)
  { # Function plotting mu, g and the minimum of g. For 2D plots xgrid is the
    # sequence of grid points along x-axis, ygrid - along y-axis.
    # draw.style is either
    # 0 for linear plot as 1D sequence
    # 1 for image() with no. of levels equal nlevels
    # 2 for contour() with no. of levels equal nlevels
    # 3 for image() and contour superposed
    # 4 for persp()

        par(mfrow=c(2,1))
   
    if (all(is.na(space)))
      {
        space<-1:length(mu)
        draw.style<-0
        dim<-1
        #warning(paste("State space is considered to be 1:length(",
        #       deparse(substitute(mu)),").",sep=""))
       }
    else
      {
        dim<-length(space[1,])
        if (dim==1)
          {
            draw.style<-0
          # warning("The only available option for 1D plot is draw.style=0")
          }
        if(dim==2 && draw.style==0)
          space<-1:length(mu)
        if(dim==2 && draw.style!=0)
          {
            if(all(is.na(xgrid))) xgrid <- as.numeric(names(table(space[,1])))
	    if(all(is.na(ygrid))) ygrid <- as.numeric(names(table(space[,2])))
          }
        if(dim>2)
          {
            draw.style<-0
            space<-1:length(mu)
      #warning("The only available option for dimensions > 2 is draw.style=0")
          }
      }

        if(draw.style==0)
          {
            len<-length(space)
            mig<-min(g)
            mag<-max(g)
            if(minim)
              {
                plot(c(space[1],space[len]),c(mig-0.2*(mag-mig),mag),
                     type="n",main="Gradient function",axes=TRUE,
                     xlab="",ylab="",...)
                lines(space,g,lwd=2,col="darkred")
                lines(space,fits,lwd=2,col="green")
              }
            else
              {
                plot(c(space[1],space[len]),c(mig,mag+0.2*(mag-mig)),
                     type="n",main="Gradient function",axes=TRUE,
                     xlab="",ylab="",...)
                lines(space,g,lwd=2,col="darkred")
                lines(space,-fits,lwd=2,col="green")
              }
            plot(space,mu,type="h",lwd=2,main="Measure",axes=TRUE,col="red",
                 xlab="",ylab="",...)
            abline(h=0)
          }
       
        if (dim==2)
          {
            G<-matrix(g,ncol=length(ygrid))
            MU<-matrix(mu,ncol=length(ygrid))
            if (draw.style==1)
              {
                if(ncolors==0)
                  image(xgrid,ygrid,G,...)
                else
                  image(xgrid,ygrid,G,col=rainbow(ncolors,start=3/6,end=5/6),...)
                title(main="Gradient function")
                if(ncolors==0)
                  image(xgrid,ygrid,MU,...)
                else
                  image(xgrid,ygrid,MU,
			col=rainbow(ncolors,start=3/6,end=5/6),...)
                title(main="Measure")
              }
            if (draw.style==2)
              {
                contour(xgrid,ygrid,G,nlevels=nlevels,...)
                title(main="Gradient function")
                contour(xgrid,ygrid,MU,nlevels=nlevels,...)
                title(main="Measure")
              }
            if (draw.style==3)
              {
                if(ncolors==0)
                  image(xgrid,ygrid,G,...)
                else
                  image(xgrid,ygrid,G,col=rainbow(ncolors,start=3/6,end=5/6),...)
                contour(xgrid,ygrid,G,nlevels=nlevels,add=TRUE,...)
                title(main="Gradient function")
                if(ncolors==0)
                  image(xgrid,ygrid,MU,...)
                else
                  image(xgrid,ygrid,MU,
			col=rainbow(ncolors,start=3/6,end=5/6),...)
                contour(xgrid,ygrid,MU,nlevels=nlevels,add=TRUE,...)
                title(main="Measure")
              }
            if (draw.style==4)
              {
                persp(xgrid,ygrid,G,theta=30, phi=30,d=3,
                      col="grey",shade=0.3,...)
                title(main="Gradient function")
                persp(xgrid,ygrid,MU,theta=30, phi=30,d=3,
                      col="cyan",shade=0.3,...)
                title(main="Measure")
              }
          } 
 }

######################## End of medea.R ################################




